@@ -24,6 +24,7 @@ from microtc.params import OPTION_DELETE, OPTION_NONE
 from microtc.utils import tweet_iterator
 from text_models.place import BoundingBox, location
 from text_models.utils import get_text
+from joblib import Parallel, delayed
 import random
 
 
@@ -410,13 +411,21 @@ class MaskedLMDataset(object):
     def dataset(self):
         return self._store
 
+    @property
+    def reader(self):
+        return self._reader
+
+    @reader.setter
+    def reader(self, reader):
+        self._reader = reader
+
     def missing(self):
         """Number of klass that has not been reached the 
         maximum number of elements
         >>> from text_models.dataset import MaskedLMDataset
         >>> ms = MaskedLMDataset(None, num_elements=64000)
         >>> ms.add(['☹'], "aa ☹ b")
-        >>> ms.add(['☹', '☺'], "aa ☹ ☺ b")
+        >>> ms.add(['☺'], "aa ☺ b")
         >>> _ = [ms.add(['☹'], "aa ☹ b") for _ in range(64010)]
         >>> ms.missing()
         1
@@ -434,22 +443,21 @@ class MaskedLMDataset(object):
         1
         >>> ms.add(['☹', '☺'], "aa ☹ ☺ b")
         >>> len(ms.dataset['☺'])
-        1
+        0
         >>> len(ms.dataset['☹'])
         1
         >>> _ = [ms.add(['☹'], "aa ☹ b") for _ in range(64010)]
         >>> len(ms.dataset['☹'])
         64000
         """
-        if len(klasses) == 0:
+        if len(klasses) != 1:
             return
         dataset = self.dataset
         num_elements = self._num_elements
-        lst = [(k, len(dataset[k])) for k in klasses]
-        lst = [(k, v) for k, v in lst if v < num_elements]
-        lst.sort(key=lambda x: x[1])
-        if len(lst):
-            dataset[lst[0][0]].append(text)
+        klass = list(klasses)[0]
+        value = dataset[klass]
+        if len(value) < num_elements:
+            value.append(text)
 
     def filter(self, text):
         """Filter RT
@@ -472,7 +480,12 @@ class MaskedLMDataset(object):
         True
         >>> ms.lang(dict(lang="en"))
         False
+        >>> ms = MaskedLMDataset(None, lang=None)
+        >>> ms.lang(dict())
+        True
         """
+        if self._lang is None:
+            return True
         return tw.get('lang', "") == self._lang
 
     def process(self, fname: str):
@@ -493,6 +506,7 @@ class MaskedLM(object):
     """Create a masked language model
     >>> from text_models.dataset import Dataset, MaskedLMDataset, MaskedLM
     >>> from EvoMSA.utils import download
+    >>> from EvoMSA.model import LabeledDataSet
     >>> from microtc.utils import load_model
     >>> from text_models.tests.test_dataset import TWEETS
     >>> emo = load_model(download("emo_En.tm"))
@@ -503,32 +517,41 @@ class MaskedLM(object):
     >>> mk = MaskedLM(mkDS)
     >>> tm = mk.textModel()
     >>> coef, intercept, labels = mk.run(tm)
+    >>> model = LabeledDataSet(textModel=tm, coef=coef, intercept=intercept, labels=labels)
+    >>> coef, intercept, labels = mk.run(tm, n_jobs=-2)
     """
     def __init__(self, masked: MaskedLMDataset) -> None:
         self._masked = masked
 
-    def run(self, tm):
-        from sklearn.svm import LinearSVC
+    def fit(self, klass, tm):
+        from sklearn.svm import LinearSVC        
+        dataset = self._masked.dataset
+        klasses = sorted(dataset.keys())
+        cnt = len(klasses) - 1
+        transform = self.transform
+        X = transform(klass, dataset[klass])
+        y = [1] * len(X)
+        nneg = max(len(X) // cnt, 1)
+        for k2 in klasses:
+            if k2 == klass:
+                continue
+            ele = dataset[k2]
+            random.shuffle(ele)
+            ele = transform(k2, ele[:nneg])
+            y += [-1] * len(ele)
+            X += ele
+        return LinearSVC().fit(tm.transform(X), y)
+        
+    def run(self, tm, n_jobs=None):
         from EvoMSA.utils import linearSVC_array
         dataset = self._masked.dataset
         klasses = sorted(dataset.keys())
         cnt = len(klasses) - 1
         transform = self.transform
-        MODELS = []
-        for klass in tqdm(klasses):
-            X = transform(klass, dataset[klass])
-            y = [1] * len(X)
-            nneg = max(len(X) // cnt, 1)
-            for k2 in klasses:
-                if k2 == klass:
-                    continue
-                ele = dataset[klass]
-                random.shuffle(ele)
-                ele = transform(k2, ele[:nneg])
-                y += [-1] * len(ele)
-                X += ele
-            m = LinearSVC().fit(tm.transform(X), y)
-            MODELS.append(m)
+        if n_jobs is not None and (n_jobs > 1 or n_jobs < 0):
+            MODELS = Parallel(n_jobs=n_jobs)(delayed(self.fit)(klass, tm) for klass in tqdm(klasses))
+        else:
+            MODELS = [self.fit(klass, tm) for klass in tqdm(klasses)]
         coef, intercept = linearSVC_array(MODELS)
         return coef, intercept, klasses
 
